package com.chelvc.framework.feign.interceptor;

import java.io.IOException;
import java.io.InputStream;

import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.JacksonUtils;
import feign.Response;
import feign.codec.ErrorDecoder;
import lombok.NonNull;

/**
 * Feign调用失败处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
public class FeignFailureInterceptor implements ErrorDecoder {
    /**
     * 反序列化请求结果
     *
     * @param response 相应结果
     * @return 请求结果
     * @throws IOException I/O异常
     */
    protected Result<?> deserialize(@NonNull Response response) throws IOException {
        try (InputStream input = response.body().asInputStream()) {
            return JacksonUtils.deserialize(input, Result.class);
        }
    }

    @Override
    public Exception decode(String method, Response response) {
        Result<?> result;
        try {
            result = this.deserialize(response);
        } catch (IOException e) {
            return e;
        }
        return new FrameworkException(result.getCode(), result.getData(), result.getMessage());
    }
}
